@@ -6,7 +6,7 @@ from datetime import datetime, timedelta
 from typing import TYPE_CHECKING
 
 from sipyconfig.comms import Command
-from sipyconfig.enums import SiCardError, SiError, com, SIAC_BEEP, siac_rev
+from sipyconfig.enums import SiCardError, SiError, com, SIAC_BEEP, siac_rev, stationmode, siacmode, mode_rev, control_number_to_mode
 from sipyconfig.utils import bytes_to_int
 
 if TYPE_CHECKING:
@@ -19,25 +19,67 @@ class Punch():
     Information about a control punch.
     Control Number and if available time code
     """
-    time: 'Optional[datetime]' = None
     control: int
+    time: 'Optional[datetime]' = None
+    control_mode: 'Optional[stationmode]' = None
+    siac_mode: 'Optional[siacmode]' = None
+    order_number: int = 0
 
-    def __init__(self, control_number: int, time: 'Optional[datetime]') -> None:
+    def __init__(
+        self,
+        control_number: int,
+        time: 'Optional[datetime]' = None,
+        control_byte: 'Optional[int]' = None, *,
+        order_number: int = 0,
+        auto_detect: bool = False
+    ) -> None:
         self.time = time
         self.control = control_number
+        self.order_number = order_number
+        if control_byte is not None:
+            try:
+                self.control_mode = stationmode(control_byte & 0x3f)
+                self.siac_mode = siacmode(control_byte & 0xf0)
+            except (ValueError, TypeError):
+                self.control_mode = stationmode.UNKNOWN
+                self.siac_mode = siacmode.NO_SIAC
+        if auto_detect and (
+            self.control_mode is None or
+            self.control_mode == stationmode.UNKNOWN
+        ):
+            self.control_mode = control_number_to_mode.get(control_number, stationmode.CONTROL)
+        if auto_detect and self.control_mode == stationmode.SIAC_SPECIAL:
+            self.control_mode = stationmode(control_number)
 
     @classmethod
-    def from_bytes(cls, control_number: int, raw_time: bytes, punchdate: int = None, reftime: 'datetime' = None) -> 'Punch':
+    def from_bytes(
+            cls,
+            control_number: int,
+            raw_time: bytes,
+            punchdate: int = None,
+            reftime: 'datetime' = None,
+            membytes: 'Optional[bytes]' = None, *,
+            auto_detect: bool = False
+        ) -> 'Punch':
         """
         construct Punch class from the byte data
         """
         punchtime = SICard.decode_time(raw_time, punchdate, reftime)
-        return Punch(control_number, punchtime)
+        control_byte = None
+        order_number = 0
+        if membytes is not None:
+            control_byte = membytes[0]
+            order_number = membytes[1]
+        return Punch(control_number, punchtime, control_byte, order_number=order_number, auto_detect=auto_detect)
 
     def __str__(self) -> str:
+        if self.control_mode is not None:
+            return f"Punch {self.control} / {mode_rev.get(self.control_mode)} at {self.time}"
         return f"Punch {self.control} at {self.time}"
 
     def __repr__(self) -> str:
+        if self.control_mode is not None and self.siac_mode is not None:
+            return f"Punch({self.control.__repr__()}, {self.time.__repr__()}, {hex(self.control_mode | self.siac_mode)})"
         return f"Punch({self.control.__repr__()}, {self.time.__repr__()})"
 
 
@@ -212,7 +254,7 @@ class SICard():
         raise NotImplementedError
 
     @classmethod
-    def from_auto_send(cls, data: bytes, station: 'SiUSBStation'):
+    def from_auto_send(cls, data: bytes, *, auto_detect_srr: bool = False):
         """create a SI Card object from the data send with auto send."""
         data_new = [x for x in data]
         if data_new[2] == 0x0f:
@@ -221,16 +263,17 @@ class SICard():
             data_new[2] = 0x0
         control_number = bytes_to_int(data_new[:2])
         si_number = cls.decode_number(data_new[2:6])
-        time = cls.decode_time(data_new[7:10], data[6])
-        # mem_pointer = bytes_to_int(data[10:])
+        time = cls.decode_time(data_new[7:9])
+        mem_pointer = data[10:]
+        control_byte = mem_pointer[0]
         card: SICard
         if si_number <= 499999:
-            card = SI5(data_new[2:6], station)
+            card = SI5(data_new[2:6])
         elif 500000 <= si_number <= 999999 or (2003000 <= si_number <= 2003999):
-            card = SI6(data_new[2:6], station)
+            card = SI6(data_new[2:6])
         else:
-            card = SI8To10.switch_types(data_new[2:6], station)
-        card.add_punch(Punch(control_number, time))
+            card = SI8To10.switch_types(data_new[2:6])
+        card.add_punch(Punch(control_number, time, control_byte, order_number=mem_pointer[1], auto_detect=auto_detect_srr))
         return card
 
     def add_punch(self, punch: Punch):
