@@ -312,7 +312,7 @@ class SiUSBStation():
         self.set_protocol_setting(CPC.AUTO_SEND, True)
         self.get_system_info()
 
-    def wait_for_si_card(self, timeout: float = None, *, no_ack: bool = False) -> Optional[SICard]:
+    def wait_for_si_card(self, timeout: float = None, *, no_ack: bool = False, auto_detect_srr: bool = False) -> Optional[SICard]:
         """waits for an SiCard to be inserted, waits a maximum of timeout (infinite if unset),
         returns a SICard object or None if timeout
         if no_ack is set True, no ACK packet will be sent to the station"""
@@ -322,7 +322,7 @@ class SiUSBStation():
         while True:
             self._currently_waiting = True
             try:
-                self.receive_command()
+                self.receive_command(auto_detect_srr=auto_detect_srr)
             except SiCardInserted:
                 if self._si_card:
                     self._si_card.read_out_data(no_ack=no_ack)
@@ -361,7 +361,7 @@ class SiUSBStation():
         self._command_tries = 0
         return ret
 
-    def receive_command(self) -> Union[Command, ACK, NOACK]:
+    def receive_command(self, *, auto_detect_srr: bool = False) -> Union[Command, ACK, NOACK]:
         """receive a command from the station, might raise SiCardEventError if SICard is inserted or removed while waiting on a response."""
         ret = receive_command(self._dev)
         if isinstance(ret, Command):
@@ -377,7 +377,7 @@ class SiUSBStation():
             elif ret.command_code == com.SI_REMOVED:
                 raise SiCardRemoved("SI-Card removed during command.")
             elif ret.command_code == com.PUNCH_DATA:
-                self._si_card = SICard.from_auto_send(ret.data, self)
+                self._si_card = SICard.from_auto_send(ret.data, auto_detect_srr=auto_detect_srr)
                 raise SiCardAutoSend("SI-Card insert with auto send enabled.")
         return ret
 
